cv.SPLS2 <-
function(x, y, penalty, fold = 10, nc, lambda, plot.it) 
{
    x <- as.matrix(x)
    n <- nrow(x)
    p <- ncol(x)
    ip <- c(1:p)
    y <- as.matrix(y)
    q <- ncol(y)
    
    x.old<-x; y.old<-y
    plot.it<-"FALSE"    


    set.seed(12345)
    foldi <- split(sample(1:n), rep(1:fold, length = n))
    mspemat <- matrix(0, length(lambda), length(nc))
    for (i in 1:length(lambda)) {
        cat(paste("lambda =", lambda[i], "\n"))
        mspemati <- matrix(0, fold, length(nc))
        for (j in 1:fold) {
            omit <- foldi[[j]]
            object <- SPLS2(x[-omit, , drop = FALSE], y[-omit, , drop = FALSE], penalty, nc = max(nc), lambda = lambda[i])
            newx <- x[omit, , drop = FALSE]
            newy<-  y[omit, , drop = FALSE]
            
            T<-object$T; W<-object$W
            for (k in nc) {
                Tk<-T[,1:k]; Wk<-W[,1:k]
                tQk<-solve(t(Tk)%*%Tk)%*%t(Tk)%*%y[-omit,, drop=FALSE]
                Beta<-Wk%*%tQk
                pred <- newx %*% Beta 
                mspemati[j, (k - min(nc) + 1)] <- mean(apply((newy - pred)^2, 2, mean))
            
            }
        }
        mspemat[i, ] <- apply(mspemati, 2, mean)
    }
    minpmse <- min(mspemat)
    rownames(mspemat) <- lambda
    colnames(mspemat) <- nc
    mspecol <- apply(mspemat, 2, min)
    msperow <- apply(mspemat, 1, min)
    nc.opt <- min(nc[mspecol == minpmse])
    lambda.opt <- max(lambda[msperow == minpmse])
    cat(paste("\nOptimal parameters: lambda = ", lambda.opt, ", ", 
        sep = ""))
    cat(paste("nc = ", nc.opt, "\n", sep = ""))
    if (plot.it) {
        heatmap.spls(t(mspemat), xlab = "nc", ylab = "lambda", main = "CV MSPE Plot", 
            coln = 16, as = "n")
    }
    rownames(mspemat) <- paste("lambda=", lambda)
    colnames(mspemat) <- paste("nc =", nc)
    cv <- list(x=x, y=y, mspemat = mspemat, lambda.opt = lambda.opt, nc.opt = nc.opt)
    invisible(cv)
}

